@@ -35,8 +35,9 @@
 #endif
 
 #include <libempathy/empathy-utils.h>
-#include <libempathy/empathy-account.h>
+#include <libempathy/empathy-idle.h>
 
+#include <telepathy-glib/account.h>
 #include <telepathy-glib/connection-manager.h>
 #include <telepathy-glib/util.h>
 #include <dbus/dbus-protocol.h>
@@ -70,6 +71,11 @@ typedef struct {
   /* An EmpathyAccountWidget can be used to either create an account or
    * modify it. When we are creating an account, this member is set to TRUE */
   gboolean creating_account;
+  /* If we are creating a new account, this member is set to TRUE once the
+   * account has been created */
+  gboolean account_created;
+
+  EmpathyIdle *idle;
 
   gboolean dispose_run;
 } EmpathyAccountWidgetPriv;
@@ -570,10 +576,11 @@ account_widget_account_enabled_cb (GObject *source_object,
     gpointer user_data)
 {
   GError *error = NULL;
-  EmpathyAccount *account = EMPATHY_ACCOUNT (source_object);
+  TpAccount *account = TP_ACCOUNT (source_object);
   EmpathyAccountWidget *widget = EMPATHY_ACCOUNT_WIDGET (user_data);
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (widget);
 
-  empathy_account_set_enabled_finish (account, res, &error);
+  tp_account_set_enabled_finish (account, res, &error);
 
   if (error != NULL)
     {
@@ -582,6 +589,7 @@ account_widget_account_enabled_cb (GObject *source_object,
     }
   else
     {
+      priv->account_created = TRUE;
       g_signal_emit (widget, signals[ACCOUNT_CREATED], 0);
     }
 }
@@ -592,7 +600,7 @@ account_widget_applied_cb (GObject *source_object,
     gpointer user_data)
 {
   GError *error = NULL;
-  EmpathyAccount *account;
+  TpAccount *account;
   EmpathyAccountSettings *settings = EMPATHY_ACCOUNT_SETTINGS (source_object);
   EmpathyAccountWidget *widget = EMPATHY_ACCOUNT_WIDGET (user_data);
   EmpathyAccountWidgetPriv *priv = GET_PRIV (widget);
@@ -613,7 +621,7 @@ account_widget_applied_cb (GObject *source_object,
       if (priv->creating_account)
         {
           /* By default, when an account is created, we enable it. */
-          empathy_account_set_enabled_async (account, TRUE,
+          tp_account_set_enabled_async (account, TRUE,
               account_widget_account_enabled_cb, widget);
         }
       else if (priv->enabled_checkbox != NULL)
@@ -629,12 +637,12 @@ account_widget_applied_cb (GObject *source_object,
                 NBTK_GTK_LIGHT_SWITCH (priv->enabled_checkbox));
 #endif
 
-          if (empathy_account_is_enabled (account) && enabled_checked)
+          if (tp_account_is_enabled (account) && enabled_checked)
             {
               /* After having applied changes to a user account, we
                * automatically reconnect it. This is done so the new
                * information entered by the user is validated on the server. */
-              empathy_account_reconnect_async (account, NULL, NULL);
+              tp_account_reconnect_async (account, NULL, NULL);
             }
         }
     }
@@ -726,7 +734,7 @@ account_widget_build_salut (EmpathyAccountWidget *self,
       "entry_jid", "jid",
       NULL);
 
-  self->ui_details->default_focus = g_strdup ("entry_nickname");
+  self->ui_details->default_focus = g_strdup ("entry_first_name");
 }
 
 static void
@@ -1047,13 +1055,13 @@ account_widget_destroy_cb (GtkWidget *widget,
 }
 
 static void
-empathy_account_widget_enabled_cb (EmpathyAccount *account,
+empathy_account_widget_enabled_cb (TpAccount *account,
       GParamSpec *spec,
       gpointer user_data)
 {
   EmpathyAccountWidget *widget = EMPATHY_ACCOUNT_WIDGET (user_data);
   EmpathyAccountWidgetPriv *priv = GET_PRIV (widget);
-  gboolean enabled = empathy_account_is_enabled (account);
+  gboolean enabled = tp_account_is_enabled (account);
 
   if (priv->enabled_checkbox != NULL)
     {
@@ -1079,7 +1087,7 @@ account_widget_switch_flipped_cb (NbtkGtkLightSwitch *sw,
 #endif /* HAVE_MOBLIN */
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (user_data);
-  EmpathyAccount *account;
+  TpAccount *account;
 #ifndef HAVE_MOBLIN
   gboolean state;
 
@@ -1089,7 +1097,7 @@ account_widget_switch_flipped_cb (NbtkGtkLightSwitch *sw,
   account = empathy_account_settings_get_account (priv->settings);
 
   /* Enable the account according to the value of the "Enabled" checkbox */
-  empathy_account_set_enabled_async (account, state, NULL, NULL);
+  tp_account_set_enabled_async (account, state, NULL, NULL);
 }
 
 static void
@@ -1144,6 +1152,26 @@ do_get_property (GObject *object,
     }
 }
 
+static void
+idle_state_change_cb (EmpathyIdle *idle,
+    GParamSpec *spec,
+    EmpathyAccountWidget *self)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+  TpConnectionPresenceType state;
+
+  state = empathy_idle_get_state (priv->idle);
+
+  if (state > TP_CONNECTION_PRESENCE_TYPE_OFFLINE)
+    {
+      g_object_set (priv->apply_button, "label", GTK_STOCK_CONNECT, NULL);
+    }
+  else
+    {
+      g_object_set (priv->apply_button, "label", GTK_STOCK_APPLY, NULL);
+    }
+}
+
 #define WIDGET(cm, proto) \
   { #cm, #proto, "empathy-account-widget-"#proto".ui", \
     account_widget_build_##proto }
@@ -1153,7 +1181,7 @@ do_constructed (GObject *obj)
 {
   EmpathyAccountWidget *self = EMPATHY_ACCOUNT_WIDGET (obj);
   EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
-  EmpathyAccount *account;
+  TpAccount *account;
   const gchar *protocol, *cm_name;
   guint i = 0;
   struct {
@@ -1242,15 +1270,38 @@ do_constructed (GObject *obj)
       GtkWidget *hbox = gtk_hbox_new (TRUE, 3);
 
       priv->cancel_button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
-      priv->apply_button = gtk_button_new_from_stock (
-        priv->creating_account ? GTK_STOCK_CONNECT : GTK_STOCK_APPLY);
 
-#ifdef HAVE_MOBLIN
       if (priv->creating_account)
-	/* Translators: this is used only when built on a moblin platform */
-	gtk_button_set_label (GTK_BUTTON (priv->apply_button),
-            _("L_og in"));
-#endif
+        {
+          TpConnectionPresenceType state;
+          priv->idle = empathy_idle_dup_singleton ();
+
+          empathy_signal_connect_weak (priv->idle, "notify::state",
+              G_CALLBACK (idle_state_change_cb), obj);
+
+          state = empathy_idle_get_state (priv->idle);
+
+          if (state > TP_CONNECTION_PRESENCE_TYPE_OFFLINE)
+            {
+              /* We are online, display a Login button */
+              GtkWidget *image;
+
+              priv->apply_button = gtk_button_new_with_mnemonic (_("L_og in"));
+              image = gtk_image_new_from_stock (GTK_STOCK_CONNECT,
+                  GTK_ICON_SIZE_BUTTON);
+              gtk_button_set_image (GTK_BUTTON (priv->apply_button), image);
+            }
+          else
+            {
+              /* We are offline, display a Save button */
+              priv->apply_button = gtk_button_new_from_stock (GTK_STOCK_SAVE);
+            }
+        }
+      else
+        {
+          /* We are editing an existing account, display an Apply button */
+          priv->apply_button = gtk_button_new_from_stock (GTK_STOCK_APPLY);
+        }
 
       gtk_box_pack_end (GTK_BOX (hbox), priv->apply_button, TRUE,
           TRUE, 3);
@@ -1295,7 +1346,7 @@ do_constructed (GObject *obj)
       guint nb_rows, nb_columns;
       gboolean is_enabled;
 
-      is_enabled = empathy_account_is_enabled (account);
+      is_enabled = tp_account_is_enabled (account);
 
 #ifndef HAVE_MOBLIN
       priv->enabled_checkbox =
@@ -1373,7 +1424,7 @@ do_dispose (GObject *obj)
 
   if (priv->settings != NULL)
     {
-      EmpathyAccount *account;
+      TpAccount *account;
       account = empathy_account_settings_get_account (priv->settings);
 
       if (account != NULL)
@@ -1386,6 +1437,12 @@ do_dispose (GObject *obj)
       priv->settings = NULL;
     }
 
+  if (priv->idle != NULL)
+    {
+      g_object_unref (priv->idle);
+      priv->idle = NULL;
+    }
+
   if (G_OBJECT_CLASS (empathy_account_widget_parent_class)->dispose != NULL)
     G_OBJECT_CLASS (empathy_account_widget_parent_class)->dispose (obj);
 }
@@ -1494,6 +1551,11 @@ empathy_account_widget_contains_pending_changes (EmpathyAccountWidget *widget)
 {
   EmpathyAccountWidgetPriv *priv = GET_PRIV (widget);
 
+  if (priv->creating_account && !priv->account_created)
+    /* We always want to warn the user if he's in the process of creating a
+     * new account which hasn't been actually created yet. */
+    return TRUE;
+
   return priv->contains_pending_changes;
 }
 
@@ -1532,3 +1594,56 @@ empathy_account_widget_new_for_protocol (EmpathyAccountSettings *settings,
 
   return self;
 }
+
+gchar *
+empathy_account_widget_get_default_display_name (EmpathyAccountWidget *self)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+  const gchar *login_id;
+  const gchar *protocol, *p;
+  gchar *default_display_name;
+
+  login_id = empathy_account_settings_get_string (priv->settings, "account");
+  protocol = empathy_account_settings_get_protocol (priv->settings);
+
+  if (login_id != NULL)
+    {
+      /* TODO: this should be done in empathy-account-widget-irc */
+      if (!tp_strdiff (protocol, "irc"))
+        {
+          const gchar* server;
+          server = empathy_account_settings_get_string (priv->settings,
+              "server");
+
+          /* To translators: The first parameter is the login id and the
+           * second one is the server. The resulting string will be something
+           * like: "MyUserName on chat.freenode.net".
+           * You should reverse the order of these arguments if the
+           * server should come before the login id in your locale.*/
+          default_display_name = g_strdup_printf (_("%1$s on %2$s"),
+              login_id, server);
+        }
+      else
+        {
+          default_display_name = g_strdup (login_id);
+        }
+
+      return default_display_name;
+    }
+
+  if ((p = empathy_protocol_name_to_display_name (protocol)) != NULL)
+    protocol = p;
+
+  if (protocol != NULL)
+    {
+      /* To translators: The parameter is the protocol name. The resulting
+       * string will be something like: "Jabber Account" */
+      default_display_name = g_strdup_printf (_("%s Account"), protocol);
+    }
+  else
+    {
+      default_display_name = g_strdup (_("New account"));
+    }
+
+  return default_display_name;
+}
