@@ -24,7 +24,7 @@
 
 #include <glib.h>
 
-#include <libmissioncontrol/mission-control.h>
+#include <telepathy-glib/enums.h>
 
 G_BEGIN_DECLS
 
@@ -64,9 +64,13 @@ void         empathy_idle_set_presence        (EmpathyIdle *idle,
 gboolean     empathy_idle_get_auto_away       (EmpathyIdle *idle);
 void         empathy_idle_set_auto_away       (EmpathyIdle *idle,
 					       gboolean     auto_away);
-gboolean     empathy_idle_get_use_nm          (EmpathyIdle *idle);
-void         empathy_idle_set_use_nm          (EmpathyIdle *idle,
-					       gboolean     use_nm);
+
+TpConnectionPresenceType empathy_idle_get_requested_presence (EmpathyIdle *idle,
+							      gchar **status,
+							      gchar **status_message);
+
+gboolean empathy_idle_account_is_just_connected (EmpathyIdle *idle,
+						 TpAccount *account);
 
 G_END_DECLS
 
