@@ -55,22 +55,20 @@ typedef struct {
 	GQueue                *pending_messages_queue;
 	gboolean               had_properties_list;
 	GPtrArray             *properties;
+	TpChannelPasswordFlags password_flags;
+	/* TRUE if we fetched the password flag of the channel or if it's not needed
+	 * (channel doesn't implement the Password interface) */
+	gboolean               got_password_flags;
 	gboolean               ready;
 } EmpathyTpChatPriv;
 
-typedef struct {
-	gchar          *name;
-	guint           id;
-	TpPropertyFlags flags;
-	GValue         *value;
-} TpChatProperty;
-
 static void tp_chat_iface_init         (EmpathyContactListIface *iface);
 
 enum {
 	PROP_0,
 	PROP_CHANNEL,
 	PROP_REMOTE_CONTACT,
+	PROP_PASSWORD_NEEDED,
 	PROP_READY,
 };
 
@@ -493,10 +491,10 @@ tp_chat_property_flags_changed_cb (TpProxy         *proxy,
 	}
 
 	for (i = 0; i < properties->len; i++) {
-		GValueArray    *prop_struct;
-		TpChatProperty *property;
-		guint           id;
-		guint           flags;
+		GValueArray           *prop_struct;
+		EmpathyTpChatProperty *property;
+		guint                  id;
+		guint                  flags;
 
 		prop_struct = g_ptr_array_index (properties, i);
 		id = g_value_get_uint (g_value_array_get_nth (prop_struct, 0));
@@ -531,10 +529,10 @@ tp_chat_properties_changed_cb (TpProxy         *proxy,
 	}
 
 	for (i = 0; i < properties->len; i++) {
-		GValueArray    *prop_struct;
-		TpChatProperty *property;
-		guint           id;
-		GValue         *src_value;
+		GValueArray           *prop_struct;
+		EmpathyTpChatProperty *property;
+		guint                  id;
+		GValue                *src_value;
 
 		prop_struct = g_ptr_array_index (properties, i);
 		id = g_value_get_uint (g_value_array_get_nth (prop_struct, 0));
@@ -597,11 +595,11 @@ tp_chat_list_properties_cb (TpProxy         *proxy,
 	ids = g_array_sized_new (FALSE, FALSE, sizeof (guint), properties->len);
 	priv->properties = g_ptr_array_sized_new (properties->len);
 	for (i = 0; i < properties->len; i++) {
-		GValueArray    *prop_struct;
-		TpChatProperty *property;
+		GValueArray           *prop_struct;
+		EmpathyTpChatProperty *property;
 
 		prop_struct = g_ptr_array_index (properties, i);
-		property = g_slice_new0 (TpChatProperty);
+		property = g_slice_new0 (EmpathyTpChatProperty);
 		property->id = g_value_get_uint (g_value_array_get_nth (prop_struct, 0));
 		property->name = g_value_dup_string (g_value_array_get_nth (prop_struct, 1));
 		property->flags = g_value_get_uint (g_value_array_get_nth (prop_struct, 3));
@@ -628,9 +626,13 @@ empathy_tp_chat_set_property (EmpathyTpChat *chat,
 			      const gchar   *name,
 			      const GValue  *value)
 {
-	EmpathyTpChatPriv *priv = GET_PRIV (chat);
-	TpChatProperty    *property;
-	guint              i;
+	EmpathyTpChatPriv     *priv = GET_PRIV (chat);
+	EmpathyTpChatProperty *property;
+	guint                  i;
+
+	if (!priv->had_properties_list) {
+		return;
+	}
 
 	for (i = 0; i < priv->properties->len; i++) {
 		property = g_ptr_array_index (priv->properties, i);
@@ -672,6 +674,36 @@ empathy_tp_chat_set_property (EmpathyTpChat *chat,
 	}
 }
 
+EmpathyTpChatProperty *
+empathy_tp_chat_get_property (EmpathyTpChat *chat,
+			      const gchar   *name)
+{
+	EmpathyTpChatPriv     *priv = GET_PRIV (chat);
+	EmpathyTpChatProperty *property;
+	guint                  i;
+
+	if (!priv->had_properties_list) {
+		return NULL;
+	}
+
+	for (i = 0; i < priv->properties->len; i++) {
+		property = g_ptr_array_index (priv->properties, i);
+		if (!tp_strdiff (property->name, name)) {
+			return property;
+		}
+	}
+
+	return NULL;
+}
+
+GPtrArray *
+empathy_tp_chat_get_properties (EmpathyTpChat *chat)
+{
+	EmpathyTpChatPriv *priv = GET_PRIV (chat);
+
+	return priv->properties;
+}
+
 static void
 tp_chat_dispose (GObject *object)
 {
@@ -727,14 +759,14 @@ tp_chat_finalize (GObject *object)
 
 	if (priv->properties) {
 		for (i = 0; i < priv->properties->len; i++) {
-			TpChatProperty *property;
+			EmpathyTpChatProperty *property;
 
 			property = g_ptr_array_index (priv->properties, i);
 			g_free (property->name);
 			if (property->value) {
 				tp_g_value_slice_free (property->value);
 			}
-			g_slice_free (TpChatProperty, property);
+			g_slice_free (EmpathyTpChatProperty, property);
 		}
 		g_ptr_array_free (priv->properties, TRUE);
 	}
@@ -750,10 +782,21 @@ tp_chat_check_if_ready (EmpathyTpChat *chat)
 {
 	EmpathyTpChatPriv *priv = GET_PRIV (chat);
 
-	if (priv->ready || priv->user == NULL ||
-	    (priv->members == NULL && priv->remote_contact == NULL)) {
+	if (priv->ready)
+		return;
+
+	if (priv->user == NULL)
+		return;
+
+	if (!priv->got_password_flags)
+		return;
+
+	/* We need either the members (room) or the remote contact (private chat).
+	 * If the chat is protected by a password we can't get these information so
+	 * consider the chat as ready so it can be presented to the user. */
+	if (!empathy_tp_chat_password_needed (chat) && priv->members == NULL &&
+	    priv->remote_contact == NULL)
 		return;
-	}
 
 	DEBUG ("Ready!");
 
@@ -1003,6 +1046,44 @@ tp_chat_got_self_contact_cb (EmpathyTpContactFactory *factory,
 	tp_chat_check_if_ready (EMPATHY_TP_CHAT (chat));
 }
 
+static void
+password_flags_changed_cb (TpChannel *channel,
+    guint added,
+    guint removed,
+    gpointer user_data,
+    GObject *weak_object)
+{
+	EmpathyTpChat *self = EMPATHY_TP_CHAT (weak_object);
+	EmpathyTpChatPriv *priv = GET_PRIV (self);
+	gboolean was_needed, needed;
+
+	was_needed = empathy_tp_chat_password_needed (self);
+
+	priv->password_flags |= added;
+	priv->password_flags ^= removed;
+
+	needed = empathy_tp_chat_password_needed (self);
+
+	if (was_needed != needed)
+		g_object_notify (G_OBJECT (self), "password-needed");
+}
+
+static void
+got_password_flags_cb (TpChannel *proxy,
+			     guint password_flags,
+			     const GError *error,
+			     gpointer user_data,
+			     GObject *weak_object)
+{
+	EmpathyTpChat *self = EMPATHY_TP_CHAT (weak_object);
+	EmpathyTpChatPriv *priv = GET_PRIV (self);
+
+	priv->got_password_flags = TRUE;
+	priv->password_flags = password_flags;
+
+	tp_chat_check_if_ready (EMPATHY_TP_CHAT (self));
+}
+
 static GObject *
 tp_chat_constructor (GType                  type,
 		     guint                  n_props,
@@ -1073,6 +1154,22 @@ tp_chat_constructor (GType                  type,
 									       G_OBJECT (chat), NULL);
 	}
 
+	/* Check if the chat is password protected */
+	if (tp_proxy_has_interface_by_id (priv->channel,
+					  TP_IFACE_QUARK_CHANNEL_INTERFACE_PASSWORD)) {
+		priv->got_password_flags = FALSE;
+
+		tp_cli_channel_interface_password_connect_to_password_flags_changed
+			(priv->channel, password_flags_changed_cb, chat, NULL,
+			 G_OBJECT (chat), NULL);
+
+		tp_cli_channel_interface_password_call_get_password_flags
+			(priv->channel, -1, got_password_flags_cb, chat, NULL, chat);
+	} else {
+		/* No Password interface, so no need to fetch the password flags */
+		priv->got_password_flags = TRUE;
+	}
+
 	return chat;
 }
 
@@ -1082,6 +1179,7 @@ tp_chat_get_property (GObject    *object,
 		      GValue     *value,
 		      GParamSpec *pspec)
 {
+	EmpathyTpChat *self = EMPATHY_TP_CHAT (object);
 	EmpathyTpChatPriv *priv = GET_PRIV (object);
 
 	switch (param_id) {
@@ -1094,6 +1192,9 @@ tp_chat_get_property (GObject    *object,
 	case PROP_READY:
 		g_value_set_boolean (value, priv->ready);
 		break;
+	case PROP_PASSWORD_NEEDED:
+		g_value_set_boolean (value, empathy_tp_chat_password_needed (self));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -1154,6 +1255,14 @@ empathy_tp_chat_class_init (EmpathyTpChatClass *klass)
 							       FALSE,
 							       G_PARAM_READABLE));
 
+	g_object_class_install_property (object_class,
+					 PROP_PASSWORD_NEEDED,
+					 g_param_spec_boolean ("password-needed",
+							       "password needed",
+							       "TRUE if a password is needed to join the channel",
+							       FALSE,
+							       G_PARAM_READABLE));
+
 	/* Signals */
 	signals[MESSAGE_RECEIVED] =
 		g_signal_new ("message-received",
@@ -1445,3 +1554,66 @@ empathy_tp_chat_acknowledge_messages (EmpathyTpChat *chat,
 	g_list_free (msgs);
 }
 
+gboolean
+empathy_tp_chat_password_needed (EmpathyTpChat *self)
+{
+	EmpathyTpChatPriv *priv = GET_PRIV (self);
+
+	return priv->password_flags & TP_CHANNEL_PASSWORD_FLAG_PROVIDE;
+}
+
+static void
+provide_password_cb (TpChannel *channel,
+				      gboolean correct,
+				      const GError *error,
+				      gpointer user_data,
+				      GObject *weak_object)
+{
+	GSimpleAsyncResult *result = user_data;
+
+	if (error != NULL) {
+		g_simple_async_result_set_from_error (result, error);
+	}
+	else if (!correct) {
+		/* The current D-Bus API is a bit weird so re-use the
+		 * AuthenticationFailed error */
+		g_simple_async_result_set_error (result, TP_ERRORS,
+						 TP_ERROR_AUTHENTICATION_FAILED, "Wrong password");
+	}
+
+	g_simple_async_result_complete (result);
+	g_object_unref (result);
+}
+
+void
+empathy_tp_chat_provide_password_async (EmpathyTpChat *self,
+						     const gchar *password,
+						     GAsyncReadyCallback callback,
+						     gpointer user_data)
+{
+	EmpathyTpChatPriv *priv = GET_PRIV (self);
+	GSimpleAsyncResult *result;
+
+	result = g_simple_async_result_new (G_OBJECT (self),
+					    callback, user_data,
+					    empathy_tp_chat_provide_password_finish);
+
+	tp_cli_channel_interface_password_call_provide_password
+		(priv->channel, -1, password, provide_password_cb, result,
+		 NULL, G_OBJECT (self));
+}
+
+gboolean
+empathy_tp_chat_provide_password_finish (EmpathyTpChat *self,
+						      GAsyncResult *result,
+						      GError **error)
+{
+	if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result),
+		error))
+		return FALSE;
+
+	g_return_val_if_fail (g_simple_async_result_is_valid (result,
+							      G_OBJECT (self), empathy_tp_chat_provide_password_finish), FALSE);
+
+	return TRUE;
+}
