@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  *
  * Authors: Xavier Claessens <xclaesse@gmail.com>
  *          Martyn Russell <martyn@imendio.com>
@@ -29,7 +29,6 @@
 #include <stdio.h>
 
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <glib.h>
 #include <glib/gi18n.h>
 
@@ -107,8 +106,8 @@ void
 empathy_chatrooms_window_show (GtkWindow *parent)
 {
 	static EmpathyChatroomsWindow *window = NULL;
-	GladeXML                     *glade;
-	gchar                        *filename;
+	GtkBuilder                    *gui;
+	gchar                         *filename;
 
 	if (window) {
 		gtk_window_present (GTK_WINDOW (window->window));
@@ -117,10 +116,8 @@ empathy_chatrooms_window_show (GtkWindow *parent)
 
 	window = g_new0 (EmpathyChatroomsWindow, 1);
 
-	filename = empathy_file_lookup ("empathy-chatrooms-window.glade", "src");
-	glade = empathy_glade_get_file (filename,
-				       "chatrooms_window",
-				       NULL,
+	filename = empathy_file_lookup ("empathy-chatrooms-window.ui", "src");
+	gui = empathy_builder_get_file (filename,
 				       "chatrooms_window", &window->window,
 				       "hbox_account", &window->hbox_account,
 				       "label_account", &window->label_account,
@@ -131,15 +128,14 @@ empathy_chatrooms_window_show (GtkWindow *parent)
 				       NULL);
 	g_free (filename);
 
-	empathy_glade_connect (glade,
-			      window,
+	empathy_builder_connect (gui, window,
 			      "chatrooms_window", "destroy", chatrooms_window_destroy_cb,
 			      "button_remove", "clicked", chatrooms_window_button_remove_clicked_cb,
 			      "button_edit", "clicked", chatrooms_window_button_edit_clicked_cb,
 			      "button_close", "clicked", chatrooms_window_button_close_clicked_cb,
 			      NULL);
 
-	g_object_unref (glade);
+	g_object_unref (gui);
 
 	g_object_add_weak_pointer (G_OBJECT (window->window), (gpointer) &window);
 
@@ -158,7 +154,7 @@ empathy_chatrooms_window_show (GtkWindow *parent)
 	empathy_account_chooser_set_filter (EMPATHY_ACCOUNT_CHOOSER (window->account_chooser),
 					    empathy_account_chooser_filter_is_connected,
 					    NULL);
-	g_object_set (window->account_chooser, 
+	g_object_set (window->account_chooser,
 		      "has-all-option", TRUE,
 		      NULL);
 	empathy_account_chooser_set_account (EMPATHY_ACCOUNT_CHOOSER (window->account_chooser), NULL);
@@ -226,7 +222,7 @@ chatrooms_window_model_setup (EmpathyChatroomsWindow *window)
 
 	gtk_tree_view_set_model (view, GTK_TREE_MODEL (store));
 
-	/* Selection */ 
+	/* Selection */
 	selection = gtk_tree_view_get_selection (view);
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
 
@@ -277,8 +273,8 @@ chatrooms_window_model_add_columns (EmpathyChatroomsWindow *window)
 
 	/* Room */
 	cell = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Room"), cell, 
-							   "text", COL_ROOM, 
+	column = gtk_tree_view_column_new_with_attributes (_("Room"), cell,
+							   "text", COL_ROOM,
 							   NULL);
 	count = gtk_tree_view_append_column (view, column);
 	gtk_tree_view_column_set_sort_column_id (column, count - 1);
@@ -297,7 +293,7 @@ chatrooms_window_model_add_columns (EmpathyChatroomsWindow *window)
 			  window);
 
 	/* Sort model */
-	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model), 0, 
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model), 0,
 					      GTK_SORT_ASCENDING);
 }
 
@@ -312,7 +308,7 @@ chatrooms_window_model_refresh_data (EmpathyChatroomsWindow *window,
 	GtkTreeIter            iter;
 	GtkTreeViewColumn     *column;
 	EmpathyAccountChooser  *account_chooser;
-	McAccount             *account;
+	TpAccount             *account;
 	GList                 *chatrooms, *l;
 
 	view = GTK_TREE_VIEW (window->treeview);
@@ -322,12 +318,12 @@ chatrooms_window_model_refresh_data (EmpathyChatroomsWindow *window,
 
 	/* Look up chatrooms */
 	account_chooser = EMPATHY_ACCOUNT_CHOOSER (window->account_chooser);
-	account = empathy_account_chooser_get_account (account_chooser);
+	account = empathy_account_chooser_dup_account (account_chooser);
 
 	chatrooms = empathy_chatroom_manager_get_chatrooms (window->manager, account);
 
 	/* Sort out columns, we only show the server column for
-	 * selected protocol types, such as Jabber. 
+	 * selected protocol types, such as Jabber.
 	 */
 	if (account) {
 		column = gtk_tree_view_get_column (view, window->room_column);
@@ -442,19 +438,9 @@ static void
 chatrooms_window_model_action_selected (EmpathyChatroomsWindow *window)
 {
 	EmpathyChatroom *chatroom;
-	GtkTreeView    *view;
-	GtkTreeModel   *model;
-
-	view = GTK_TREE_VIEW (window->treeview);
-	model = gtk_tree_view_get_model (view);
 
+	/* FIXME: This is still not implemented since Gossip fork */
 	chatroom = chatrooms_window_model_get_selected (window);
-	if (!chatroom) {
-		return;
-	}
-
-	//empathy_edit_chatroom_dialog_show (GTK_WINDOW (window->window), chatroom);
-
 	g_object_unref (chatroom);
 }
 
@@ -502,13 +488,8 @@ chatrooms_window_button_edit_clicked_cb (GtkWidget             *widget,
 {
 	EmpathyChatroom *chatroom;
 
+	/* FIXME: This is still not implemented since Gossip fork */
 	chatroom = chatrooms_window_model_get_selected (window);
-	if (!chatroom) {
-		return;
-	}
-
-	//empathy_edit_chatroom_dialog_show (GTK_WINDOW (window->window), chatroom);
-
 	g_object_unref (chatroom);
 }
 
@@ -525,15 +506,15 @@ chatrooms_window_chatroom_added_cb (EmpathyChatroomManager *manager,
 				    EmpathyChatroomsWindow *window)
 {
 	EmpathyAccountChooser *account_chooser;
-	McAccount            *account;
+	TpAccount             *account;
 
 	account_chooser = EMPATHY_ACCOUNT_CHOOSER (window->account_chooser);
-	account = empathy_account_chooser_get_account (account_chooser);
+	account = empathy_account_chooser_dup_account (account_chooser);
 
 	if (!account) {
 		chatrooms_window_model_add (window, chatroom, FALSE);
 	} else {
-		if (empathy_account_equal (account, empathy_chatroom_get_account (chatroom))) {
+		if (account == empathy_chatroom_get_account (chatroom)) {
 			chatrooms_window_model_add (window, chatroom, FALSE);
 		}
 
